//+-----------------------------------------------------------------+
//| Filtered Price, May 2014 Version                                |
//+-----------------------------------------------------------------+
//---- Indicator Settings
#property indicator_chart_window       // Locates indicator in Chart window.
#property indicator_buffers 1          // Number of Buffers
#property indicator_color1  RoyalBlue
#property indicator_width1  3
//---- Input Parameters
extern string TimeFrame  = "Current";
extern int    Price      = PRICE_CLOSE;
extern double Factor     = 2.0;
extern double Step       = 0.0;
//---- Buffers
double FP   [];
double Trend[];
double SMinP[];
double SMaxP[];
double SMinS[];
double SMaxS[];
//---- Time Frame Variables
string Name  ;
bool   RBars ;
int    TFrame;

//+-------------------------------------------------------+
//| Filtered Price Initialization Function                |
//+-------------------------------------------------------+
int init()
  {
//----  Filtered Price Buffers
   IndicatorBuffers(6);
   SetIndexBuffer(0,FP   );SetIndexLabel(0,"Filtered Price");
   SetIndexBuffer(1,Trend);
   SetIndexBuffer(2,SMinP);
   SetIndexBuffer(3,SMaxP);
   SetIndexBuffer(4,SMinS);
   SetIndexBuffer(5,SMaxS);
 
   Name  = WindowExpertName();
   RBars = TimeFrame == "RBars";
   if (RBars)  return(0);
   TFrame = STTFrame(TimeFrame);
   // Indicator Name
   IndicatorShortName(TFTString(TFrame)+" Filtered Price, Price "+Price);
//--- Initialization is done.
   return(0);
  }

//+-------------------------------------------------------+
//| Filtered Price Deinitialization Function              |
//+-------------------------------------------------------+
int deinit()
  {
   return(0);
  }

//+-------------------------------------------------------+
//| Filtered Price Main Function                          |
//+-------------------------------------------------------+
int start()
  {
   int i;
   int CountedBars = IndicatorCounted();
   if (CountedBars < 0)  return(-1);
   if (CountedBars > 0)  CountedBars--;
   int Limit = MathMin(Bars-CountedBars,Bars-1);
//---- Multiple Time Frame Adjustment
   if (RBars)
     {
      FP[0] = MathMin(Limit+1,Bars-1);
      return(0);
     }
   if (TFrame != Period())
     {
      Limit = MathMax(Limit,MathMin(Bars-1,iCustom(Symbol(),TFrame,Name,"RBars",0,0)*TFrame/Period()));
      for (i=Limit; i>=0; i--)
        {
         int   y  = iBarShift(Symbol(),TFrame,Time[i]);
         FP   [i] = iCustom  (Symbol(),TFrame,Name,"",Price,Factor,Step,0,y);
        }
      return(0);
     }

//---- Filtered Price Iteration Function
   for (i=Limit; i>= 0; i--)
     {
      FP[i] = iMA(Symbol(),Period(),1,0,MODE_SMA,Price,i);
      // Step Handling
      Trend[i] = Trend[i+1];
      SMinP[i] = SMinP[i+1];
      SMaxP[i] = SMaxP[i+1];
      SMinS[i] = SMinS[i+1];
      SMaxS[i] = SMaxS[i+1];
      SMaxP[i] = FP[i]+Factor*Step;
      SMinP[i] = FP[i]-Factor*Step;
      if (FP[i] > SMaxS[i])  Trend[i] =  1;
      if (FP[i] < SMinS[i])  Trend[i] = -1;
      if (Trend[i] > 0 && SMinP[i] < SMinS[i])  SMinP[i] = SMinS[i];
      if (Trend[i] < 0 && SMaxP[i] > SMaxS[i])  SMaxP[i] = SMaxS[i];
      if (Trend[i] > 0)  FP[i] = SMinP[i]+Step;
      if (Trend[i] < 0)  FP[i] = SMaxP[i]-Step;
      SMinS[i] = SMinP[i];
      SMaxS[i] = SMaxP[i];
     }
   return(0);
  }

//+-------------------------------------------------------+
//| Filtered Price Complementary Functions                |
//+-------------------------------------------------------+
//---- Timer
string TFSTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    ITFTable[] = {1,5,15,30,60,240,1440,10080,43200};
int STTFrame(string TFS)
  {
   TFS = StringUpperCase(TFS);
   for (int i=ArraySize(ITFTable)-1; i>=0; i--)
   if (TFS == TFSTable[i] || TFS == ""+ITFTable[i])  return(MathMax(ITFTable[i],Period()));
   return(Period());
  }

string TFTString(int TF)
  {
   for (int i=ArraySize(ITFTable)-1; i>=0; i--)
   if (TF == ITFTable[i])  return(TFSTable[i]);
   return("");
  }

string StringUpperCase(string STR)
  {
   string s = STR;
   for (int length=StringLen(STR)-1; length>=0; length--)
     {
      int Char = StringGetChar(s,length);
      if ((Char > 96 && Char < 123) || (Char > 223 && Char < 256))  s = StringSetChar(s,length,Char-32);
      else if (Char > -33 && Char < 0)  s = StringSetChar(s,length,Char+224);
     }
   return(s);
  }
//+-------------------------------------------------------+